@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2009-2013 Sourcefire, Inc.
 
  *  Authors: Török Edvin, Kevin Lin
@@ -42,11 +42,19 @@ struct cli_exe_section;
 struct DISASM_RESULT;
 #endif
 
-  /**
-\group_pe
-   * Invalid RVA specified
-   */
+#ifdef __CLAMBC__
+enum {
+    /**
+     * \group_pe
+     * Invalid RVA specified
+     */
+    PE_INVALID_RVA = 0xFFFFFFFF
+};
+#endif
+
+#ifndef __CLAMBC__
 #define PE_INVALID_RVA 0xFFFFFFFF
+#endif
 
 /**
 \group_config
@@ -54,12 +62,12 @@ struct DISASM_RESULT;
  */
 enum BytecodeKind {
     /** generic bytecode, not tied a specific hook */
-    BC_GENERIC=0,
+    BC_GENERIC = 0,
     /** triggered at startup, only one is allowed per ClamAV startup */
-    BC_STARTUP=1,
-    _BC_START_HOOKS=256,
+    BC_STARTUP      = 1,
+    _BC_START_HOOKS = 256,
     /** executed on a logical trigger */
-    BC_LOGICAL=256,
+    BC_LOGICAL = 256,
     /** specifies a PE unpacker, executed on PE files on a logical trigger */
     BC_PE_UNPACKER,
     /** specifies a PDF hook, executes at a predetermined point of PDF parsing for PDF files */
@@ -70,6 +78,10 @@ enum BytecodeKind {
     /** specifies a PRECLASS hook, executes at the end of file property collection and
       * operates on the original file targeted for property collection */
     BC_PRECLASS,
+    /** specifies an ELF unpacker, executed on ELF files on a logical trigger */
+    BC_ELF_UNPACKER,
+    /** specifies an Mach-O unpacker, executed on Mach-O files on a logical trigger */
+    BC_MACHO_UNPACKER,
     _BC_LAST_HOOK
 };
 
@@ -78,51 +90,68 @@ enum BytecodeKind {
  * LibClamAV functionality level constants
  */
 enum FunctionalityLevels {
-    FUNC_LEVEL_096       = 51, /* LibClamAV release 0.96.0: bytecode engine released */
+    FUNC_LEVEL_096       = 51, /**< LibClamAV release 0.96.0: bytecode engine released */
     FUNC_LEVEL_096_dev   = 52,
-    FUNC_LEVEL_096_1     = 53, /* LibClamAV release 0.96.1: logical signature use of VI/macros
+    FUNC_LEVEL_096_1     = 53, /**< LibClamAV release 0.96.1: logical signature use of VI/macros
                                 * requires this minimum functionality level */
     FUNC_LEVEL_096_1_dev = 54,
-    FUNC_LEVEL_096_2     = 54, /* LibClamAV release 0.96.2: PDF Hooks require this minimum level */
+    FUNC_LEVEL_096_2     = 54, /**< LibClamAV release 0.96.2: PDF Hooks require this minimum level */
     FUNC_LEVEL_096_2_dev = 55,
-    FUNC_LEVEL_096_3     = 55, /* LibClamAV release 0.96.3: BC_PE_ALL bytecodes require this minimum level */
-    FUNC_LEVEL_096_4     = 56, /* LibClamAV release 0.96.4: minimum recommended engine version, older versions
-                                * have quadratic load time */
-    FUNC_LEVEL_096_5     = 58, /* LibClamAV release 0.96.5 */
-    FUNC_LEVEL_097       = 60, /* LibClamAV release 0.97.0: older bytecodes may incorrectly use 57 */
-    FUNC_LEVEL_097_1     = 61, /* LibClamAV release 0.97.1 */
-    FUNC_LEVEL_097_2     = 62, /* LibClamAV release 0.97.2 */
-    FUNC_LEVEL_097_3     = 63, /* LibClamAV release 0.97.3 */ /*last bcc changes as former team resigns*/
-    FUNC_LEVEL_097_4     = 64, /* LibClamAV release 0.97.4 */
-    FUNC_LEVEL_097_5     = 65, /* LibClamAV release 0.97.5 */
-    FUNC_LEVEL_097_6     = 67, /* LibClamAV release 0.97.6 */
-    FUNC_LEVEL_097_7     = 68, /* LibClamAV release 0.97.7 */
-    FUNC_LEVEL_097_8     = 69, /* LibClamAV release 0.97.8 */
-    FUNC_LEVEL_098_1     = 76, /* LibClamAV release 0.98.1 */ /*last syncing to clamav*/
-    FUNC_LEVEL_098_2     = 77, /* LibClamAV release 0.98.2 */
-    FUNC_LEVEL_098_3     = 77, /* LibClamAV release 0.98.3 */
-    FUNC_LEVEL_098_4     = 77, /* LibClamAV release 0.98.4 */
-    FUNC_LEVEL_098_5     = 79, /* LibClamAV release 0.98.5: JSON reading API requires this minimum level */
-    FUNC_LEVEL_098_6     = 79, /* LibClamAV release 0.98.6 */
-    FUNC_LEVEL_098_7     = 80, /* LibClamAV release 0.98.7: BC_PRECLASS bytecodes require minimum level */
-    FUNC_LEVEL_099       = 81, /* LibClamAV release 0.99, 0.99-beta1(.1-.5), 0.99-beta2 */
-    FUNC_LEVEL_099_1     = 82, /* LibClamAV release 0.99.1 */
-    FUNC_LEVEL_099_2     = 82, /* LibClamAV release 0.99.2 */
-    FUNC_LEVEL_099_3     = 84, /* LibClamAV release 0.99.3 */
-    FUNC_LEVEL_099_4     = 85, /* LibClamAV release 0.99.4 */
-    FUNC_LEVEL_0100_0_BETA = 90, /* LibClamAV beta release 0.100.0-beta */
-    FUNC_LEVEL_0100_0    = 91, /* LibClamAV release 0.100.0, 0.100.0-rc */
+    FUNC_LEVEL_096_3     = 55, /**< LibClamAV release 0.96.3: BC_PE_ALL bytecodes require this minimum level */
+    FUNC_LEVEL_096_4     = 56, /**< LibClamAV release 0.96.4: minimum recommended engine version, older versions have quadratic load time */
+    FUNC_LEVEL_096_5     = 58, /**< LibClamAV release 0.96.5 */
+
+    FUNC_LEVEL_097   = 60, /**< LibClamAV release 0.97.0: older bytecodes may incorrectly use 57 */
+    FUNC_LEVEL_097_1 = 61, /**< LibClamAV release 0.97.1 */
+    FUNC_LEVEL_097_2 = 62, /**< LibClamAV release 0.97.2 */
+    FUNC_LEVEL_097_3 = 63, /**< LibClamAV release 0.97.3: last bcc changes as former team resigns*/
+    FUNC_LEVEL_097_4 = 64, /**< LibClamAV release 0.97.4 */
+    FUNC_LEVEL_097_5 = 65, /**< LibClamAV release 0.97.5 */
+    FUNC_LEVEL_097_6 = 67, /**< LibClamAV release 0.97.6 */
+    FUNC_LEVEL_097_7 = 68, /**< LibClamAV release 0.97.7 */
+    FUNC_LEVEL_097_8 = 69, /**< LibClamAV release 0.97.8 */
+
+    FUNC_LEVEL_098   = 74, /**< LibClamAV release 0.98.0 */
+    FUNC_LEVEL_098_1 = 76, /**< LibClamAV release 0.98.1: last syncing to clamav*/
+    FUNC_LEVEL_098_2 = 77, /**< LibClamAV release 0.98.2 */
+    FUNC_LEVEL_098_3 = 77, /**< LibClamAV release 0.98.3 */
+    FUNC_LEVEL_098_4 = 77, /**< LibClamAV release 0.98.4 */
+    FUNC_LEVEL_098_5 = 79, /**< LibClamAV release 0.98.5: JSON reading API requires this minimum level */
+    FUNC_LEVEL_098_6 = 79, /**< LibClamAV release 0.98.6 */
+    FUNC_LEVEL_098_7 = 80, /**< LibClamAV release 0.98.7: BC_PRECLASS bytecodes require minimum level */
+
+    FUNC_LEVEL_099   = 81, /**< LibClamAV release 0.99.0 */
+    FUNC_LEVEL_099_1 = 82, /**< LibClamAV release 0.99.1 */
+    FUNC_LEVEL_099_2 = 82, /**< LibClamAV release 0.99.2: not a typo, the FLEVEL did not change */
+    FUNC_LEVEL_099_3 = 84, /**< LibClamAV release 0.99.3 */
+    FUNC_LEVEL_099_4 = 85, /**< LibClamAV release 0.99.4 */
+
+    FUNC_LEVEL_0100_BETA = 90, /**< LibClamAV release 0.100.0-BETA */
+    FUNC_LEVEL_0100      = 91, /**< LibClamAV release 0.100.0 */
     FUNC_LEVEL_0100_1    = 92, /**< LibClamAV release 0.100.1 */
     FUNC_LEVEL_0100_2    = 93, /**< LibClamAV release 0.100.2 */
     FUNC_LEVEL_0100_3    = 94, /**< LibClamAV release 0.100.3 */
-    FUNC_LEVEL_0101_0_BETA = 100, /* LibClamAV beta release 0.101.0-beta */
-    FUNC_LEVEL_0101_0    = 101, /* LibClamAV release 0.101.0, 0.101.0-rc */
-    FUNC_LEVEL_0101_1    = 102, /* LibClamAV release 0.101.1 */
-    FUNC_LEVEL_0101_2    = 102, /* LibClamAV release 0.101.2 */
-    FUNC_LEVEL_0101_3    = 102, /* LibClamAV release 0.101.3 */
-    FUNC_LEVEL_0101_4    = 105, /* LibClamAV release 0.101.4 */
-    FUNC_LEVEL_0101_5    = 106, /* LibClamAV release 0.101.5 */
-    FUNC_LEVEL_100       = 255 /* future release candidate */
+
+    FUNC_LEVEL_0101_BETA = 100, /**< LibClamAV release 0.101.0-BETA */
+    FUNC_LEVEL_0101      = 101, /**< LibClamAV release 0.101.0 */
+    FUNC_LEVEL_0101_1    = 102, /**< LibClamAV release 0.101.1 */
+    FUNC_LEVEL_0101_2    = 102, /**< LibClamAV release 0.101.2 */
+    FUNC_LEVEL_0101_3    = 102, /**< LibClamAV release 0.101.3 */
+    FUNC_LEVEL_0101_4    = 105, /**< LibClamAV release 0.101.4 */
+    FUNC_LEVEL_0101_5    = 106, /**< LibClamAV release 0.101.5 */
+
+    FUNC_LEVEL_0102_BETA = 110, /**< LibClamAV release 0.102.0-BETA */
+    FUNC_LEVEL_0102      = 111, /**< LibClamAV release 0.102.0 */
+    FUNC_LEVEL_0102_1    = 112, /**< LibClamAV release 0.102.1 */
+
+    FUNC_LEVEL_0103_BETA = 120, /**< LibClamAV release 0.103.0-BETA */
+    FUNC_LEVEL_0103      = 121, /**< LibClamAV release 0.103.0 */
+    FUNC_LEVEL_0103_1    = 122, /**< LibClamAV release 0.103.1 */
+    FUNC_LEVEL_0103_2    = 123, /**< LibClamAV release 0.103.2 */
+    FUNC_LEVEL_0103_3    = 124, /**< LibClamAV release 0.103.3 */
+    FUNC_LEVEL_0103_4    = 125, /**< LibClamAV release 0.103.4 */
+    FUNC_LEVEL_0103_5    = 126, /**< LibClamAV release 0.103.5 */
+    FUNC_LEVEL_0103_6    = 127, /**< LibClamAV release 0.103.6 */
 };
 
 /**
@@ -142,7 +171,7 @@ enum pdf_phase {
  * PDF flags
  */
 enum pdf_flag {
-    BAD_PDF_VERSION=0,      /* */
+    BAD_PDF_VERSION = 0,    /* */
     BAD_PDF_HEADERPOS,      /* */
     BAD_PDF_TRAILER,        /* */
     BAD_PDF_TOOMANYOBJS,    /* */
@@ -170,7 +199,7 @@ enum pdf_flag {
  * PDF obj flags
  */
 enum pdf_objflags {
-    OBJ_STREAM=0,        /* */
+    OBJ_STREAM = 0,      /* */
     OBJ_DICT,            /* */
     OBJ_EMBEDDED_FILE,   /* */
     OBJ_FILTER_AH,       /* */
@@ -202,53 +231,87 @@ enum pdf_objflags {
  * JSON types
  */
 enum bc_json_type {
-    JSON_TYPE_NULL=0,    /* */
-    JSON_TYPE_BOOLEAN,   /* */
-    JSON_TYPE_DOUBLE,    /* */
-    JSON_TYPE_INT,       /* */
-    JSON_TYPE_OBJECT,    /* */
-    JSON_TYPE_ARRAY,     /* */
-    JSON_TYPE_STRING     /* */
+    JSON_TYPE_NULL = 0, /* */
+    JSON_TYPE_BOOLEAN,  /* */
+    JSON_TYPE_DOUBLE,   /* */
+    JSON_TYPE_INT,      /* */
+    JSON_TYPE_OBJECT,   /* */
+    JSON_TYPE_ARRAY,    /* */
+    JSON_TYPE_STRING    /* */
+};
+
+/**
+\group_adt
+  * LZMA return codes
+  */
+enum lzma_returncode {
+    LZMA_RESULT_OK         = 0, /* Function completed successfully */
+    LZMA_RESULT_DATA_ERROR = 1, /* The LZMA stream contained invalid data */
+    LZMA_STREAM_END        = 2  /* The LZMA stream ended unexpectedly */
+};
+
+/**
+\group adt
+ * Bzip2 return codes
+ */
+enum bzip2_returncode {
+    BZIP2_OK               = 0, /* Function returned without error */
+    BZIP2_SEQUENCE_ERROR   = -1,
+    BZIP2_PARAM_ERROR      = -2,
+    BZIP2_MEM_ERROR        = -3,
+    BZIP2_DATA_ERROR       = -4,
+    BZIP2_DATA_ERROR_MAGIC = -5,
+    BZIP2_IO_ERROR         = -6,
+    BZIP2_UNEXPECTED_EOF   = -7,
+    BZIP2_OUTBUFF_FULL     = -8,
+    BZIP2_CONFIG_ERROR     = -9,
+    BZIP2_RUN_OK           = 1,
+    BZIP2_FLUSH_OK         = 2,
+    BZIP2_FINISH_OK        = 3,
+    BZIP2_STREAM_END       = 4
 };
 
 /**
 \group_engine
  * Scan option flag values for engine_scan_options(). *DEPRECATED*
  */
-#define CL_SCAN_RAW                     0x0
-#define CL_SCAN_ARCHIVE                 0x1
-#define CL_SCAN_MAIL                    0x2
-#define CL_SCAN_OLE2                    0x4
-#define CL_SCAN_BLOCKENCRYPTED          0x8
-#define CL_SCAN_HTML                    0x10
-#define CL_SCAN_PE                      0x20
-#define CL_SCAN_BLOCKBROKEN             0x40
-#define CL_SCAN_MAILURL                 0x80  /* deprecated circa 2009 */
-#define CL_SCAN_BLOCKMAX                0x100
-#define CL_SCAN_ALGORITHMIC             0x200
-//#define UNUSED                        0x400
-#define CL_SCAN_PHISHING_BLOCKSSL       0x800 /* ssl mismatches, not ssl by itself*/
-#define CL_SCAN_PHISHING_BLOCKCLOAK     0x1000
-#define CL_SCAN_ELF                     0x2000
-#define CL_SCAN_PDF                     0x4000
-#define CL_SCAN_STRUCTURED              0x8000
-#define CL_SCAN_STRUCTURED_SSN_NORMAL   0x10000
-#define CL_SCAN_STRUCTURED_SSN_STRIPPED 0x20000
-#define CL_SCAN_PARTIAL_MESSAGE         0x40000
-#define CL_SCAN_HEURISTIC_PRECEDENCE    0x80000
-#define CL_SCAN_BLOCKMACROS             0x100000
-#define CL_SCAN_ALLMATCHES              0x200000
-#define CL_SCAN_SWF                     0x400000
-#define CL_SCAN_PARTITION_INTXN         0x800000
-#define CL_SCAN_XMLDOCS                 0x1000000
-#define CL_SCAN_HWP3                    0x2000000
-//#define UNUSED                        0x4000000
-//#define UNUSED                        0x8000000
-#define CL_SCAN_FILE_PROPERTIES         0x10000000
-//#define UNUSED                        0x20000000
-#define CL_SCAN_PERFORMANCE_INFO        0x40000000 /* Collect performance timings */
-#define CL_SCAN_INTERNAL_COLLECT_SHA    0x80000000 /* Enables hash output in sha-collect builds - for internal use only */
-
+// clang-format off
+enum {
+   CL_SCAN_RAW                      = 0x0,
+   CL_SCAN_ARCHIVE                  = 0x1,
+   CL_SCAN_MAIL                     = 0x2,
+   CL_SCAN_OLE2                     = 0x4,
+   CL_SCAN_BLOCKENCRYPTED           = 0x8,
+   CL_SCAN_HTML                     = 0x10,
+   CL_SCAN_PE                       = 0x20,
+   CL_SCAN_BLOCKBROKEN              = 0x40,
+   CL_SCAN_MAILURL                  = 0x80,  /* deprecated circa 2009 */
+   CL_SCAN_BLOCKMAX                 = 0x100,
+   CL_SCAN_ALGORITHMIC              = 0x200,
+  // UNUSED                         = 0x400,
+   CL_SCAN_PHISHING_BLOCKSSL        = 0x800, /* ssl mismatches, not ssl by itself*/
+   CL_SCAN_PHISHING_BLOCKCLOAK      = 0x1000,
+   CL_SCAN_ELF                      = 0x2000,
+   CL_SCAN_PDF                      = 0x4000,
+   CL_SCAN_STRUCTURED               = 0x8000,
+   CL_SCAN_STRUCTURED_SSN_NORMAL    = 0x10000,
+   CL_SCAN_STRUCTURED_SSN_STRIPPED  = 0x20000,
+   CL_SCAN_PARTIAL_MESSAGE          = 0x40000,
+   CL_SCAN_HEURISTIC_PRECEDENCE     = 0x80000,
+   CL_SCAN_BLOCKMACROS              = 0x100000,
+   CL_SCAN_ALLMATCHES               = 0x200000,
+   CL_SCAN_SWF                      = 0x400000,
+   CL_SCAN_PARTITION_INTXN          = 0x800000,
+   CL_SCAN_XMLDOCS                  = 0x1000000,
+   CL_SCAN_HWP3                     = 0x2000000,
+  // UNUSED                         = 0x4000000,
+  // UNUSED                         = 0x8000000,
+   CL_SCAN_FILE_PROPERTIES          = 0x10000000,
+  // UNUSED                         = 0x20000000,
+   CL_SCAN_PERFORMANCE_INFO         = 0x40000000, /* Collect performance timings */
+   CL_SCAN_INTERNAL_COLLECT_SHA     = 0x80000000 /* Enables hash output in sha-collect builds - for internal use only */
+};
+// clang-format on
 
 #ifdef __CLAMBC__
 
@@ -303,14 +366,14 @@ uint32_t test1(uint32_t a, uint32_t b);
  * @param[out] data pointer to buffer where data is read into
  * @return amount read.
  */
-int32_t read(uint8_t *data, int32_t size);
+int32_t read(uint8_t* data, int32_t size);
 
 /**
 \group_file
  */
 enum {
     /**set file position to specified absolute position */
-    SEEK_SET=0,
+    SEEK_SET = 0,
     /**set file position relative to current position */
     SEEK_CUR,
     /**set file position relative to file end*/
@@ -327,7 +390,7 @@ enum {
  * byte
  * @return amount of bytes successfully written
  */
-int32_t write(uint8_t *data, int32_t size);
+int32_t write(uint8_t* data, int32_t size);
 
 /**
 \group_file
@@ -346,7 +409,7 @@ int32_t seek(int32_t pos, uint32_t whence);
  * @param[in] len length of the virusname
  * @return 0
  */
-uint32_t setvirusname(const uint8_t *name, uint32_t len);
+uint32_t setvirusname(const uint8_t* name, uint32_t len);
 
 /**
 \group_debug
@@ -355,7 +418,7 @@ uint32_t setvirusname(const uint8_t *name, uint32_t len);
  * @param[in] len length of message to print
  * @return 0
  */
-uint32_t debug_print_str(const uint8_t *str, uint32_t len);
+uint32_t debug_print_str(const uint8_t* str, uint32_t len);
 
 /**
 \group_debug
@@ -443,7 +506,7 @@ uint32_t test2(uint32_t a);
  * @return  0 - success
  * @return -1 - failure
  */
-int32_t get_pe_section(struct cli_exe_section *section, uint32_t num);
+int32_t get_pe_section(struct cli_exe_section* section, uint32_t num);
 
 /**
 \group_file
@@ -541,7 +604,7 @@ int32_t hashset_empty(int32_t id);
  * @param[in] size size of buffer
  * @return ID of newly created buffer_pipe
  */
-int32_t  buffer_pipe_new(uint32_t size);
+int32_t buffer_pipe_new(uint32_t size);
 
 /**
   \group_adt
@@ -550,7 +613,7 @@ int32_t  buffer_pipe_new(uint32_t size);
   * @param[in] pos starting position of pipe input in current file
   * @return ID of newly created buffer_pipe
   */
-int32_t  buffer_pipe_new_fromfile(uint32_t pos);
+int32_t buffer_pipe_new_fromfile(uint32_t pos);
 
 /**
 \group_adt
@@ -571,7 +634,7 @@ uint32_t buffer_pipe_read_avail(int32_t id);
   * specified amount
   */
 //uint8_t *buffer_pipe_read_get(int32_t id, uint32_t amount);
-const uint8_t *buffer_pipe_read_get(int32_t id, uint32_t amount);
+const uint8_t* buffer_pipe_read_get(int32_t id, uint32_t amount);
 
 /**
 \group_adt
@@ -580,7 +643,7 @@ const uint8_t *buffer_pipe_read_get(int32_t id, uint32_t amount);
   * @param[in] amount amount of bytes to move read cursor
   * @return 0 on success
   */
-int32_t  buffer_pipe_read_stopped(int32_t id, uint32_t amount);
+int32_t buffer_pipe_read_stopped(int32_t id, uint32_t amount);
 
 /**
 \group_adt
@@ -600,7 +663,7 @@ uint32_t buffer_pipe_write_avail(int32_t id);
   * @return pointer to write buffer, or NULL if requested amount
   * is more than what is available in the buffer
   */
-uint8_t *buffer_pipe_write_get(int32_t id, uint32_t size);
+uint8_t* buffer_pipe_write_get(int32_t id, uint32_t size);
 
 /**
 \group_adt
@@ -609,7 +672,7 @@ uint8_t *buffer_pipe_write_get(int32_t id, uint32_t size);
   * @param[in] amount amount of bytes to move write cursor
   * @return 0 on success
   */
-int32_t  buffer_pipe_write_stopped(int32_t id, uint32_t amount);
+int32_t buffer_pipe_write_stopped(int32_t id, uint32_t amount);
 
 /**
 \group_adt
@@ -620,7 +683,7 @@ int32_t  buffer_pipe_write_stopped(int32_t id, uint32_t amount);
   * @param[in] id ID of buffer_pipe
   * @return 0 on success
   */
-int32_t  buffer_pipe_done(int32_t id);
+int32_t buffer_pipe_done(int32_t id);
 
 /**
 \group_adt
@@ -789,7 +852,7 @@ int32_t atoi(const uint8_t* str, int32_t size);
   * @param[in] len length of \p str
   * @return 0
   */
-uint32_t debug_print_str_start(const uint8_t *str, uint32_t len);
+uint32_t debug_print_str_start(const uint8_t* str, uint32_t len);
 
 /**
 \group_debug
@@ -799,7 +862,7 @@ uint32_t debug_print_str_start(const uint8_t *str, uint32_t len);
   * @param[in] len length of \p str
   * @return 0
   */
-uint32_t debug_print_str_nonl(const uint8_t *str, uint32_t len);
+uint32_t debug_print_str_nonl(const uint8_t* str, uint32_t len);
 
 /**
 \group_string
@@ -830,7 +893,7 @@ int32_t map_new(int32_t keysize, int32_t valuesize);
   * @return 1 - if key didn't exist before
   * @return <0 - if ksize doesn't match keysize specified at table creation
   */
-int32_t map_addkey(const uint8_t *key, int32_t ksize, int32_t id);
+int32_t map_addkey(const uint8_t* key, int32_t ksize, int32_t id);
 
 /**
 \group_adt
@@ -841,7 +904,7 @@ int32_t map_addkey(const uint8_t *key, int32_t ksize, int32_t id);
   * @return 0 - if update was successful
   * @return <0 - if there is no last key
   */
-int32_t map_setvalue(const uint8_t *value, int32_t vsize, int32_t id);
+int32_t map_setvalue(const uint8_t* value, int32_t vsize, int32_t id);
 
 /**
 \group_adt
@@ -910,7 +973,7 @@ int32_t map_done(int32_t id);
   * match_pos + \p len < \p maxpos
   * @return offset in the current file if match is found, -1 otherwise
   */
-int32_t file_find_limit(const uint8_t *data, uint32_t len, int32_t maxpos);
+int32_t file_find_limit(const uint8_t* data, uint32_t len, int32_t maxpos);
 
 /* ------------- Engine Query ----------------------------------------------- */
 /**
@@ -978,7 +1041,7 @@ int32_t input_switch(int32_t extracted_file);
   * @param[in] len - size of \p env
   * @return 0
   */
-uint32_t get_environment(struct cli_environment *env, uint32_t len);
+uint32_t get_environment(struct cli_environment* env, uint32_t len);
 
 /**
 \group_env
@@ -991,7 +1054,7 @@ uint32_t get_environment(struct cli_environment *env, uint32_t len);
   * @return 1 - JIT disabled
   * @return 2 - fully disabled
   */
-uint32_t disable_bytecode_if(const int8_t *reason, uint32_t len, uint32_t cond);
+uint32_t disable_bytecode_if(const int8_t* reason, uint32_t len, uint32_t cond);
 
 /**
 \group_env
@@ -1018,7 +1081,7 @@ uint32_t disable_jit_if(const int8_t* reason, uint32_t len, uint32_t cond);
   * @return 1 - lhs > rhs
   */
 int32_t version_compare(const uint8_t* lhs, uint32_t lhs_len,
-                    const uint8_t* rhs, uint32_t rhs_len);
+                        const uint8_t* rhs, uint32_t rhs_len);
 
 /**
 \group_env
@@ -1084,7 +1147,7 @@ uint32_t pdf_getobjsize(int32_t objidx);
  * @return NULL - invalid objidx/amount
  * @return pointer - pointer to original object */
 //uint8_t *pdf_getobj(int32_t objidx, uint32_t amount);
-const uint8_t *pdf_getobj(int32_t objidx, uint32_t amount);
+const uint8_t* pdf_getobj(int32_t objidx, uint32_t amount);
 
 /**
 \group_pdf
@@ -1276,42 +1339,120 @@ int32_t json_get_int(int32_t objid);
   * Check if any given scan option is enabled.
   * Returns non-zero if the following named options are set:
   *
-  * "general allmatch"                - all-match mode is enabled
-  * "general collect metadata"        - --gen-json is enabled
-  * "general heuristics"              - --gen-json is enabled
+  * "general allmatch"                  - all-match mode is enabled
+  * "general collect metadata"          - --gen-json is enabled
+  * "general heuristics"                - --gen-json is enabled
   *
-  * "parse archive"                   - archive parsing is enabled
-  * "parse pdf"                       - pdf parsing is enabled
-  * "parse swf"                       - swf parsing is enabled
-  * "parse hwp3"                      - hwp3 parsing is enabled
-  * "parse xmldocs"                   - xmldocs parsing is enabled
-  * "parse mail"                      - mail parsing is enabled
-  * "parse ole2"                      - ole2 parsing is enabled
-  * "parse html"                      - html parsing is enabled
-  * "parse pe"                        - pe parsing is enabled
+  * "parse archive"                     - archive parsing is enabled
+  * "parse pdf"                         - pdf parsing is enabled
+  * "parse swf"                         - swf parsing is enabled
+  * "parse hwp3"                        - hwp3 parsing is enabled
+  * "parse xmldocs"                     - xmldocs parsing is enabled
+  * "parse mail"                        - mail parsing is enabled
+  * "parse ole2"                        - ole2 parsing is enabled
+  * "parse html"                        - html parsing is enabled
+  * "parse pe"                          - pe parsing is enabled
   *
-  * "heuristic precedence"            - heuristic signatures are set to take precedence
-  * "heuristic broken"                - broken pe heuristic is enabled
-  * "heuristic exceeds max"           - heuristic for when max settings are exceeded is enabled
-  * "heuristic phishing ssl mismatch" - phishing ssl mismatch heuristic is enabled
-  * "heuristic phishing cloak"        - phishing cloak heuristic is enabled
-  * "heuristic macros"                - macros heuristic is enabled
-  * "heuristic encrypted"             - encrypted heuristic is enabled
-  * "heuristic partition intxn"       - macpartition intxnros heuristic is enabled
-  * "heuristic structured"            - structured heuristic is enabled
-  * "heuristic structured ssn normal" - structured ssn normal heuristic is enabled
+  * "heuristic precedence"              - heuristic signatures are set to take precedence
+  * "heuristic broken"                  - broken pe heuristic is enabled
+  * "heuristic exceeds max"             - heuristic for when max settings are exceeded is enabled
+  * "heuristic phishing ssl mismatch"   - phishing ssl mismatch heuristic is enabled
+  * "heuristic phishing cloak"          - phishing cloak heuristic is enabled
+  * "heuristic macros"                  - macros heuristic is enabled
+  * "heuristic encrypted"               - encrypted heuristic is enabled
+  * "heuristic partition intersection"  - macpartition intersectionros heuristic is enabled
+  * "heuristic structured"              - structured heuristic is enabled
+  * "heuristic structured ssn normal"   - structured ssn normal heuristic is enabled
   * "heuristic structured ssn stripped" - structured ssn stripped heuristic is enabled
   *
-  * "mail partial message"            - parsing of partial mail messages is enabled
+  * "mail partial message"              - parsing of partial mail messages is enabled
   *
-  * "dev collect sha"                 - --dev-collect-hashes is enabled
-  * "dev collect performance info"    - --dev-performance is enabled
+  * "dev collect sha"                   - --dev-collect-hashes is enabled
+  * "dev collect performance info"      - --dev-performance is enabled
   *
   * @param[in] scan_options enum value for desired scan option category.
   * @return CL_SCAN_<OPTION>_* flags
   */
-uint32_t engine_scan_options_ex(const uint8_t *option_name, uint32_t name_len);
+uint32_t engine_scan_options_ex(const uint8_t* option_name, uint32_t name_len);
 
 /* ----------------- END 0.101 APIs ---------------------------------- */
+/* ----------------- BEGIN 0.103 APIs -------------------------------- */
+
+/**
+\group_adt
+  * Initializes LZMA data structures for decompressing data
+  * 'from_buffer' and writing uncompressed data 'to_buffer'.
+  * This function expects the LZMA data to be prefixed with an 'LZMA_ALONE' header:
+  * - One byte of lzma parameters lc, lp and pb converted into a byte value like this: lc + 9 * (5 * pb + lp).
+  *   lc The number of high bits of the previous byte to use as a context for literal encoding.
+  *   lp The number of low bits of the dictionary position to include in literal_pos_state.
+  *   pb The number of low bits of the dictionary position to include in pos_state.
+  * - Four bytes of dictionary size. In case of doubt you can set this to zero.
+  * - Eight bytes of uncompressed size. Can be set to -1 if the size is unknown
+  *   and the lzma stream is terminated with an end marker.
+  * @param[in] from_buffer ID of buffer_pipe to read compressed data from
+  * @param[in] to_buffer ID of buffer_pipe to write decompressed data to
+  * @return ID of newly created lzma data structure, <0 on failure.
+  */
+int32_t lzma_init(int32_t from, int32_t to);
+
+/**
+\group_adt
+  * Decompress all available data in the input buffer, and write to output buffer.
+  * Stops when the input buffer becomes empty, or write buffer becomes full.
+  * This function can be called repeatedly on success after filling the input
+  * buffer, and flushing the output buffer.
+  * The lzma stream is done processing when 0 bytes are available from output
+  * buffer, and input buffer is not empty.
+  * @param[in] id ID of lzma data structure.
+  * @return 0 on success, lzma error code otherwise.
+  */
+int32_t lzma_process(int32_t id);
+
+/**
+\group_adt
+  * Deallocates lzma data structure.
+  * Using the lzma data structure after this will result in an error.
+  * All lzma data structures are automatically deallocated when bytecode
+  * finishes execution.
+  * @param[in] id ID of lzma data structure
+  * @return 0 on success.
+  */
+int32_t lzma_done(int32_t id);
+
+/**
+\group_adt
+  * Initializes Bzip2 data structures for decompressing data
+  * 'from_buffer' and writing uncompressed data 'to_buffer'.
+  * @param[in] from_buffer ID of buffer_pipe to read compressed data from
+  * @param[in] to_buffer ID of buffer_pipe to write decompressed data to
+  * @return ID of newly created bzip2 data structure, <0 on failure.
+  */
+int32_t bzip2_init(int32_t from, int32_t to);
+
+/**
+\group_adt
+  * Decompress all available data in the input buffer, and write to output buffer.
+  * Stops when the input buffer becomes empty, or write buffer becomes full.
+  * This function can be called repeatedly on success after filling the input
+  * buffer, and flushing the output buffer.
+  * The bzip2 stream is done processing when 0 bytes are available from output
+  * buffer, and input buffer is not empty.
+  * @param[in] id ID of lzma data structure.
+  * @return 0 on success, bzip2 error code otherwise.
+  */
+int32_t bzip2_process(int32_t id);
+
+/**
+\group_adt
+  * Deallocates bzip2 data structure.
+  * Using the bzip2 data structure after this will result in an error.
+  * All bzip2 data structures are automatically deallocated when bytecode
+  * finishes execution.
+  * @param[in] id ID of bzip2 data structure
+  * @return 0 on success.
+  */
+int32_t bzip2_done(int32_t id);
+/* ----------------- END 0.103 APIs ---------------------------------- */
 #endif
 #endif
