@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
@@ -38,21 +38,23 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdbool.h>
+
+#ifdef HAVE_JSON
+#include <json.h>
+#endif
 
 #include "clamav.h"
 #include "dconf.h"
 #include "filetypes.h"
 #include "fmap.h"
-#include "libclamunrar_iface/unrar_iface.h"
 #include "regex/regex.h"
 #include "bytecode.h"
 #include "bytecode_api.h"
 #include "events.h"
 #include "crtmgr.h"
 
-#ifdef HAVE_JSON
-#include "json-c/json.h"
-#endif
+#include "unrar_iface.h"
 
 #ifdef HAVE_YARA
 #include "yara_clam.h"
@@ -71,7 +73,7 @@
  * in re-enabling affected modules.
  */
 
-#define CL_FLEVEL 114
+#define CL_FLEVEL 127
 #define CL_FLEVEL_DCONF CL_FLEVEL
 #define CL_FLEVEL_SIGTOOL CL_FLEVEL
 
@@ -79,38 +81,70 @@ extern uint8_t cli_debug_flag;
 extern uint8_t cli_always_gen_section_hash;
 
 /*
- * CLI_ISCONTAINED(bb, bb_size, sb, sb_size) checks if sb (sub buffer) is contained
- * within bb (buffer).
+ * CLI_ISCONTAINED(bb, bb_size, sb, sb_size) checks if sb (small buffer) is
+ * within bb (big buffer).
  *
  * bb and sb are pointers (or offsets) for the main buffer and the
  * sub-buffer respectively, and bb_size and sb_size are their sizes
  *
  * The macro can be used to protect against wraps.
  */
-#define CLI_ISCONTAINED(bb, bb_size, sb, sb_size)                                           \
-    (                                                                                       \
-        (size_t)(bb_size) > 0 && (size_t)(sb_size) > 0 &&                                   \
-        (size_t)(sb_size) <= (size_t)(bb_size) &&                                           \
-        (ptrdiff_t)(sb) >= (ptrdiff_t)(bb) &&                                               \
-        (ptrdiff_t)(sb) + (ptrdiff_t)(sb_size) <= (ptrdiff_t)(bb) + (ptrdiff_t)(bb_size) && \
-        (ptrdiff_t)(sb) + (ptrdiff_t)(sb_size) > (ptrdiff_t)(bb) &&                         \
-        (ptrdiff_t)(sb) < (ptrdiff_t)(bb) + (ptrdiff_t)(bb_size))
+#define CLI_ISCONTAINED(bb, bb_size, sb, sb_size)                            \
+    ((size_t)(bb_size) > 0 && (size_t)(sb_size) > 0 &&                       \
+     (size_t)(sb_size) <= (size_t)(bb_size) &&                               \
+     (size_t)(sb) >= (size_t)(bb) &&                                         \
+     (size_t)(sb) + (size_t)(sb_size) <= (size_t)(bb) + (size_t)(bb_size) && \
+     (size_t)(sb) + (size_t)(sb_size) > (size_t)(bb) &&                      \
+     (size_t)(sb) < (size_t)(bb) + (size_t)(bb_size))
 
 /*
- * CLI_ISCONTAINED2(bb, bb_size, sb, sb_size) checks if sb (sub buffer) is contained
- * within bb (buffer).
+ * CLI_ISCONTAINED_0_TO(bb_size, sb, sb_size) checks if sb (small offset) is
+ * within bb (big offset) where the big offset always starts at 0.
  *
- * CLI_ISCONTAINED2 is the same as CLI_ISCONTAINED except that it allows for sub-
- * buffers with sb_size == 0.
+ * bb and sb are offsets for the main buffer and the
+ * sub-buffer respectively, and bb_size and sb_size are their sizes
+ *
+ * The macro can be used to protect against wraps.
+ *
+ * CLI_ISCONTAINED_0_TO is the same as CLI_ISCONTAINED except that `bb` is gone
+ * and assumed ot be zero.
  */
-#define CLI_ISCONTAINED2(bb, bb_size, sb, sb_size)                                          \
-    (                                                                                       \
-        (size_t)(bb_size) > 0 && (size_t)(sb_size) >= 0 &&                                  \
-        (size_t)(sb_size) <= (size_t)(bb_size) &&                                           \
-        (ptrdiff_t)(sb) >= (ptrdiff_t)(bb) &&                                               \
-        (ptrdiff_t)(sb) + (ptrdiff_t)(sb_size) <= (ptrdiff_t)(bb) + (ptrdiff_t)(bb_size) && \
-        (ptrdiff_t)(sb) + (ptrdiff_t)(sb_size) >= (ptrdiff_t)(bb) &&                        \
-        (ptrdiff_t)(sb) < (ptrdiff_t)(bb) + (ptrdiff_t)(bb_size))
+#define CLI_ISCONTAINED_0_TO(bb_size, sb, sb_size)            \
+    ((size_t)(bb_size) > 0 && (size_t)(sb_size) > 0 &&        \
+     (size_t)(sb_size) <= (size_t)(bb_size) &&                \
+     (size_t)(sb) + (size_t)(sb_size) <= (size_t)(bb_size) && \
+     (size_t)(sb) < (size_t)(bb_size))
+
+/*
+ * CLI_ISCONTAINED_2(bb, bb_size, sb, sb_size) checks if sb (small buffer) is
+ * within bb (big buffer).
+ *
+ * CLI_ISCONTAINED_2 is the same as CLI_ISCONTAINED except that it allows for
+ * small-buffers with sb_size == 0.
+ */
+#define CLI_ISCONTAINED_2(bb, bb_size, sb, sb_size)                          \
+    ((size_t)(bb_size) > 0 &&                                                \
+     (size_t)(sb_size) <= (size_t)(bb_size) &&                               \
+     (size_t)(sb) >= (size_t)(bb) &&                                         \
+     (size_t)(sb) + (size_t)(sb_size) <= (size_t)(bb) + (size_t)(bb_size) && \
+     (size_t)(sb) + (size_t)(sb_size) >= (size_t)(bb) &&                     \
+     (size_t)(sb) <= (size_t)(bb) + (size_t)(bb_size))
+
+/*
+ * CLI_ISCONTAINED_2(bb, bb_size, sb, sb_size) checks if sb (small buffer) is
+ * within bb (big buffer).
+ *
+ * CLI_ISCONTAINED_2 is the same as CLI_ISCONTAINED except that it allows for
+ * small-buffers with sb_size == 0.
+ *
+ * CLI_ISCONTAINED_2_0_TO is the same as CLI_ISCONTAINED_2 except that `bb` is gone
+ * and assumed ot be zero.
+ */
+#define CLI_ISCONTAINED_2_0_TO(bb_size, sb, sb_size)          \
+    ((size_t)(bb_size) > 0 &&                                 \
+     (size_t)(sb_size) <= (size_t)(bb_size) &&                \
+     (size_t)(sb) + (size_t)(sb_size) <= (size_t)(bb_size) && \
+     (size_t)(sb) <= (size_t)(bb_size))
 
 #define CLI_MAX_ALLOCATION (182 * 1024 * 1024)
 
@@ -141,33 +175,39 @@ typedef struct bitset_tag {
     unsigned long length;
 } bitset_t;
 
-typedef struct cli_ctx_container_tag {
+typedef struct recursion_level_tag {
     cli_file_t type;
     size_t size;
-    unsigned char flag;
-} cli_ctx_container;
-#define CONTAINER_FLAG_VALID 0x01
+    cl_fmap_t *fmap;                      /* The fmap for this layer. This used to be in an array in the ctx. */
+    uint32_t recursion_level_buffer;      /* Which buffer layer in scan recursion. */
+    uint32_t recursion_level_buffer_fmap; /* Which fmap layer in this buffer. */
+    bool is_normalized_layer;             /* Indicates that the layer should be skipped when checking container and intermediate types. */
+} recursion_level_t;
+// #define CONTAINER_FLAG_VALID 0x01
 
 /* internal clamav context */
 typedef struct cli_ctx_tag {
-    char *target_filepath;
-    const char *sub_filepath;
+    char *target_filepath;    /**< (optional) The filepath of the original scan target. */
+    const char *sub_filepath; /**< (optional) The filepath of the current file being parsed. May be a temp file. */
+    char *sub_tmpdir;         /**< The directory to store tmp files at this recursion depth. */
     const char **virname;
     unsigned int num_viruses;
     unsigned long int *scanned;
     const struct cli_matcher *root;
     const struct cl_engine *engine;
-    unsigned long scansize;
+    uint64_t scansize;
     struct cl_scan_options *options;
-    unsigned int recursion;
     unsigned int scannedfiles;
     unsigned int found_possibly_unwanted;
     unsigned int corrupted_input;
     unsigned int img_validate;
-    cli_ctx_container *containers; /* set container type after recurse */
+    recursion_level_t *recursion_stack; /* Array of recursion levels used as a stack. */
+    uint32_t recursion_stack_size;      /* stack size must == engine->max_recursion_level */
+    uint32_t recursion_level;           /* Index into recursion_stack; current fmap recursion level from start of scan. */
+    fmap_t *fmap;                       /* Pointer to current fmap in recursion_stack, varies with recursion depth. For convenience. */
+    bool next_layer_is_normalized;      /* Indicate that the next fmap pushed to the stack is normalized and should be ignored when checking container/intermediate types */
     unsigned char handlertype_hash[16];
     struct cli_dconf *dconf;
-    fmap_t **fmap;
     bitset_t *hook_lsig_matches;
     void *cb_ctx;
     cli_events_t *perf;
@@ -179,7 +219,8 @@ typedef struct cli_ctx_tag {
     struct json_object *wrkproperty;
 #endif
     struct timeval time_limit;
-    int limit_exceeded;
+    bool limit_exceeded; /* To guard against alerting on limits exceeded more than once, or storing that in the JSON metadata more than once. */
+    bool abort_scan;     /* So we can guarantee a scan is aborted, even if CL_ETIMEOUT/etc. status is lost in the scan recursion stack. */
 } cli_ctx;
 
 #define STATS_ANON_UUID "5b585e8f-3be5-11e3-bf0b-18037319526c"
@@ -285,15 +326,15 @@ struct cl_engine {
     uint64_t engine_options;
 
     /* Limits */
-    uint32_t maxscantime; /* Time limit (in milliseconds) */
-    uint64_t maxscansize; /* during the scanning of archives this size
+    uint32_t maxscantime;         /* Time limit (in milliseconds) */
+    uint64_t maxscansize;         /* during the scanning of archives this size
 				           * will never be exceeded
 				           */
-    uint64_t maxfilesize; /* compressed files will only be decompressed
+    uint64_t maxfilesize;         /* compressed files will only be decompressed
 				           * and scanned up to this size
 				           */
-    uint32_t maxreclevel; /* maximum recursion level for archives */
-    uint32_t maxfiles;    /* maximum number of files to be scanned
+    uint32_t max_recursion_level; /* maximum recursion level for archives */
+    uint32_t maxfiles;            /* maximum number of files to be scanned
 				           * within a single archive
 				           */
     /* This is for structured data detection.  You can set the minimum
@@ -428,7 +469,7 @@ struct cl_settings {
     uint32_t maxscantime;
     uint64_t maxscansize;
     uint64_t maxfilesize;
-    uint32_t maxreclevel;
+    uint32_t max_recursion_level;
     uint32_t maxfiles;
     uint32_t min_cc_count;
     uint32_t min_ssn_count;
@@ -487,7 +528,7 @@ extern cl_unrar_error_t (*cli_unrar_extract_file)(void *hArchive, const char *de
 extern cl_unrar_error_t (*cli_unrar_skip_file)(void *hArchive);
 extern void (*cli_unrar_close)(void *hArchive);
 
-extern int have_rar;
+extern LIBCLAMAV_EXPORT int have_rar;
 
 #define SCAN_ALLMATCHES (ctx->options->general & CL_SCAN_GENERAL_ALLMATCHES)
 #define SCAN_COLLECT_METADATA (ctx->options->general & CL_SCAN_GENERAL_COLLECT_METADATA)
@@ -507,6 +548,7 @@ extern int have_rar;
 #define SCAN_PARSE_PE (ctx->options->parse & CL_SCAN_PARSE_PE)
 
 #define SCAN_HEURISTIC_BROKEN (ctx->options->heuristic & CL_SCAN_HEURISTIC_BROKEN)
+#define SCAN_HEURISTIC_BROKEN_MEDIA (ctx->options->heuristic & CL_SCAN_HEURISTIC_BROKEN_MEDIA)
 #define SCAN_HEURISTIC_EXCEEDS_MAX (ctx->options->heuristic & CL_SCAN_HEURISTIC_EXCEEDS_MAX)
 #define SCAN_HEURISTIC_PHISHING_SSL_MISMATCH (ctx->options->heuristic & CL_SCAN_HEURISTIC_PHISHING_SSL_MISMATCH)
 #define SCAN_HEURISTIC_PHISHING_CLOAK (ctx->options->heuristic & CL_SCAN_HEURISTIC_PHISHING_CLOAK)
@@ -635,15 +677,114 @@ static inline void cli_writeint32(void *offset, uint32_t value)
 }
 #endif
 
+/**
+ * @brief Append an alert.
+ *
+ * An FP-check will verify that the file is not whitelisted.
+ * The whitelist check does not happen before the scan because file whitelisting
+ * is so infrequent that such action would be detrimental to performance.
+ *
+ * TODO: Replace implementation with severity scale, and severity threshold
+ * wherein signatures that do not meet the threshold are documented in JSON
+ * metadata but do not halt the scan.
+ *
+ * @param ctx       The scan context.
+ * @param virname   The alert name.
+ * @return cl_error_t CL_VIRUS if scan should be halted due to an alert, CL_CLEAN if scan should continue.
+ */
 cl_error_t cli_append_virus(cli_ctx *ctx, const char *virname);
+
+/**
+ * @brief Append a PUA (low severity) alert.
+ *
+ * This function will return CLEAN unless in all-match or Heuristic-precedence
+ * modes. The intention is for the scan to continue in case something more
+ * malicious is found.
+ *
+ * TODO: Replace implementation with severity scale, and severity threshold
+ * wherein signatures that do not meet the threshold are documented in JSON
+ * metadata but do not halt the scan.
+ *
+ * BUG: In normal scan mode (see above), the alert is not FP-checked!
+ *
+ * @param ctx       The scan context.
+ * @param virname   The alert name.
+ * @return cl_error_t CL_VIRUS if scan should be halted due to an alert, CL_CLEAN if scan should continue.
+ */
+cl_error_t cli_append_possibly_unwanted(cli_ctx *ctx, const char *virname);
+
 const char *cli_get_last_virus(const cli_ctx *ctx);
 const char *cli_get_last_virus_str(const cli_ctx *ctx);
 void cli_virus_found_cb(cli_ctx *ctx);
 
-void cli_set_container(cli_ctx *ctx, cli_file_t type, size_t size);
-cli_file_t cli_get_container(cli_ctx *ctx, int index);
-size_t cli_get_container_size(cli_ctx *ctx, int index);
-cli_file_t cli_get_container_intermediate(cli_ctx *ctx, int index);
+/**
+ * @brief Push a new fmap onto our scan recursion stack.
+ *
+ * May fail if we exceed max recursion depth.
+ *
+ * @param ctx           The scanning context.
+ * @param map           The fmap for the new layer.
+ * @param type          The file type. May be CL_TYPE_ANY if unknown. Can change it later with cli_recursion_stack_change_type().
+ * @param is_new_buffer true if the fmap represents a new buffer/file, and not some window into an existing fmap.
+ * @return cl_error_t   CL_SUCCESS if successful, else CL_EMAXREC if exceeding the max recursion depth.
+ */
+cl_error_t cli_recursion_stack_push(cli_ctx *ctx, cl_fmap_t *map, cli_file_t type, bool is_new_buffer);
+
+/**
+ * @brief Pop off a layer of our scan recursion stack.
+ *
+ * Returns the fmap for the popped layer. Does NOT funmap() the fmap for you.
+ *
+ * @param ctx           The scanning context.
+ * @return cl_fmap_t*   A pointer to the fmap for the popped layer, may return NULL instead if the stack is empty.
+ */
+cl_fmap_t *cli_recursion_stack_pop(cli_ctx *ctx);
+
+/**
+ * @brief Re-assign the type for the current layer.
+ *
+ * @param ctx   The scanning context.
+ * @param type  The new file type.
+ */
+void cli_recursion_stack_change_type(cli_ctx *ctx, cli_file_t type);
+
+/**
+ * @brief Get the type of a specific layer.
+ *
+ * Ignores normalized layers internally.
+ *
+ * For index:
+ *  0 == the outermost (bottom) layer of the stack.
+ *  1 == the first layer (probably never explicitly used).
+ * -1 == the present innermost (top) layer of the stack.
+ * -2 == the parent layer (or "container"). That is, the second from the top of the stack.
+ *
+ * @param ctx           The scanning context.
+ * @param index         Desired index, will be converted internally as though the normalized layers were stripped out. Don't think too had about it. Or do. ¯\_(ツ)_/¯
+ * @return cli_file_t   The type of the requested layer,
+ *                      or returns CL_TYPE_ANY if a negative layer is requested,
+ *                      or returns CL_TYPE_IGNORED if requested layer too high.
+ */
+cli_file_t cli_recursion_stack_get_type(cli_ctx *ctx, int index);
+
+/**
+ * @brief Get the size of a specific layer.
+ *
+ * Ignores normalized layers internally.
+ *
+ * For index:
+ *  0 == the outermost (bottom) layer of the stack.
+ *  1 == the first layer (probably never explicitly used).
+ * -1 == the present innermost (top) layer of the stack.
+ * -2 == the parent layer (or "container"). That is, the second from the top of the stack.
+ *
+ * @param ctx           The scanning context.
+ * @param index         Desired index, will be converted internally as though the normalized layers were stripped out. Don't think too had about it. Or do. ¯\_(ツ)_/¯
+ * @return cli_file_t   The size of the requested layer,
+ *                      or returns the size of the whole file if a negative layer is requested,
+ *                      or returns 0 if requested layer too high.
+ */
+size_t cli_recursion_stack_get_size(cli_ctx *ctx, int index);
 
 /* used by: spin, yc (C) aCaB */
 #define __SHIFTBITS(a) (sizeof(a) << 3)
@@ -757,11 +898,16 @@ const char *cli_gettmpdir(void);
 /**
  * @brief Sanitize a relative path, so it cannot have a negative depth.
  *
- * Caller is responsible for freeing the filename.
+ * Caller is responsible for freeing the sanitized filepath.
+ * The optioal sanitized_filebase output param is a pointer into the filepath,
+ * if set, and does not need to be freed.
  *
- * @return char* filename or NULL.
+ * @param filepath                  The filepath to sanitize
+ * @param filepath_len              The length of the filepath
+ * @param[out] sanitized_filebase   Pointer to the basename portion of the sanitized filepath. (optional)
+ * @return char*
  */
-char *cli_sanitize_filepath(const char *filepath, size_t filepath_len);
+char *cli_sanitize_filepath(const char *filepath, size_t filepath_len, char **sanitized_filebase);
 
 /**
  * @brief Generate tempfile filename (no path) with a random MD5 hash.
@@ -772,6 +918,30 @@ char *cli_sanitize_filepath(const char *filepath, size_t filepath_len);
  */
 char *cli_genfname(const char *prefix);
 
+/**
+ * @brief Generate a full tempfile filepath with a provided the name.
+ *
+ * Caller is responsible for freeing the filename.
+ * If the dir is not provided, the engine->tmpdir will be used.
+ *
+ * @param dir 	 Alternative directory. (optional)
+ * @return char* filename or NULL.
+ */
+char *cli_newfilepath(const char *dir, const char *fname);
+
+/**
+ * @brief Generate a full tempfile filepath with a provided the name.
+ *
+ * Caller is responsible for freeing the filename.
+ * If the dir is not provided, the engine->tmpdir will be used.
+ *
+ * @param dir        Alternative temp directory (optional).
+ * @param prefix  	 (Optional) Base filename for new file.
+ * @param[out] name  Allocated filepath, must be freed by caller.
+ * @param[out] fd    File descriptor of open temp file.
+ */
+cl_error_t cli_newfilepathfd(const char *dir, char *fname, char **name, int *fd);
+
 /**
  * @brief Generate a full tempfile filepath with a random MD5 hash and prefix the name, if provided.
  *
@@ -820,15 +990,26 @@ void cli_bitset_free(bitset_t *bs);
 int cli_bitset_set(bitset_t *bs, unsigned long bit_offset);
 int cli_bitset_test(bitset_t *bs, unsigned long bit_offset);
 const char *cli_ctime(const time_t *timep, char *buf, const size_t bufsize);
-void cli_check_blockmax(cli_ctx *, int);
+void cli_append_virus_if_heur_exceedsmax(cli_ctx *, char *);
 cl_error_t cli_checklimits(const char *, cli_ctx *, unsigned long, unsigned long, unsigned long);
-cl_error_t cli_updatelimits(cli_ctx *, unsigned long);
+
+/**
+ * @brief Call before scanning a file to determine if we should scan it, skip it, or abort the entire scanning process.
+ *
+ * If the verdict is CL_SUCCESS, then this function increments the # of scanned files, and increments the amount of scanned data.
+ * If the verdict is that a limit has been exceeded, then ctx->
+ *
+ * @param ctx       The scanning context.
+ * @param needed    The size of the file we're considering scanning.
+ * @return cl_error_t CL_SUCCESS if we're good to keep scanning else an error status.
+ */
+cl_error_t cli_updatelimits(cli_ctx *ctx, size_t needed);
+
 unsigned long cli_getsizelimit(cli_ctx *, unsigned long);
 int cli_matchregex(const char *str, const char *regex);
 void cli_qsort(void *a, size_t n, size_t es, int (*cmp)(const void *, const void *));
 void cli_qsort_r(void *a, size_t n, size_t es, int (*cmp)(const void *, const void *, const void *), void *arg);
 cl_error_t cli_checktimelimit(cli_ctx *ctx);
-cl_error_t cli_append_possibly_unwanted(cli_ctx *ctx, const char *virname);
 
 /* symlink behaviour */
 #define CLI_FTW_FOLLOW_FILE_SYMLINK 0x01
@@ -857,37 +1038,58 @@ struct cli_ftw_cbdata {
     void *data;
 };
 
-/*
- * return CL_BREAK to break out without an error, CL_SUCCESS to continue,
- * or any CL_E* to break out due to error.
+/**
+ * @brief Callback to process each file in a file tree walk (FTW).
+ *
  * The callback is responsible for freeing filename when it is done using it.
+ *
  * Note that callback decides if directory traversal should continue
  * after an error, we call the callback with reason == error,
  * and if it returns CL_BREAK we break.
+ *
+ * Return:
+ * - CL_BREAK to break out without an error,
+ * - CL_SUCCESS to continue,
+ * - any CL_E* to break out due to error.
  */
-typedef int (*cli_ftw_cb)(STATBUF *stat_buf, char *filename, const char *path, enum cli_ftw_reason reason, struct cli_ftw_cbdata *data);
+typedef cl_error_t (*cli_ftw_cb)(STATBUF *stat_buf, char *filename, const char *path, enum cli_ftw_reason reason, struct cli_ftw_cbdata *data);
 
-/*
- * returns 1 if the path should be skipped and 0 otherwise
- * uses callback data
+/**
+ * @brief Callback to determine if a path in a file tree walk (FTW) should be skipped.
+ * Has access to the same callback data as the main FTW callback function (above).
+ *
+ * Return:
+ * - 1 if the path should be skipped (i.e. to not call the callback for the given path),
+ * - 0 if the path should be processed (i.e. to call the callback for the given path).
  */
 typedef int (*cli_ftw_pathchk)(const char *path, struct cli_ftw_cbdata *data);
 
-/*
- * returns
- *  CL_SUCCESS if it traversed all files and subdirs
- *  CL_BREAK if traversal has stopped at some point
- *  CL_E* if error encountered during traversal and we had to break out
+/**
+ * @brief Traverse a file path, calling the callback function on each file
+ * within if the pathchk() check allows for it. Will skip certain file types:
+ * -
+ *
  * This is regardless of virus found/not, that is the callback's job to store.
  * Note that the callback may dispatch async the scan, so that when cli_ftw
  * returns we don't know the infected/notinfected status of the directory yet!
+ *
  * Due to this if the callback scans synchronously it should store the infected
  * status in its cbdata.
  * This works for both files and directories. It stats the path to determine
  * which one it is.
  * If it is a file, it simply calls the callback once, otherwise recurses.
+ *
+ * @param base      The top level directory (or file) path to be processed
+ * @param flags     A bitflag field for the CLI_FTW_* flag options (see above)
+ * @param maxdepth  The max recursion depth.
+ * @param callback  The cli_ftw_cb callback to invoke on each file AND directory.
+ * @param data      Callback data for the callback function.
+ * @param pathchk   A function used to determine if the callback should be run on the given file.
+ * @return cl_error_t CL_SUCCESS if it traversed all files and subdirs
+ * @return cl_error_t CL_BREAK if traversal has stopped at some point
+ * @return cl_error_t CL_E* if error encountered during traversal and we had to break out
  */
-int cli_ftw(char *base, int flags, int maxdepth, cli_ftw_cb callback, struct cli_ftw_cbdata *data, cli_ftw_pathchk pathchk);
+cl_error_t cli_ftw(char *base, int flags, int maxdepth, cli_ftw_cb callback, struct cli_ftw_cbdata *data, cli_ftw_pathchk pathchk);
 
 const char *cli_strerror(int errnum, char *buf, size_t len);
 
@@ -903,4 +1105,26 @@ const char *cli_strerror(int errnum, char *buf, size_t len);
  */
 cl_error_t cli_get_filepath_from_filedesc(int desc, char **filepath);
 
+/**
+ * @brief   Attempt to get the real path of a provided path (evaluating symlinks).
+ *
+ * Caller is responsible for free'ing the file path.
+ * On posix systems this just calls realpath() under the hood.
+ * On Win32, it opens a handle and uses cli_get_filepath_from_filedesc()
+ * to get the real path.
+ *
+ * @param desc          A file path to evaluate.
+ * @param char*         [out] A malloced string containing the real path.
+ * @return cl_error_t   CL_SUCCESS if found, else an error code.
+ */
+cl_error_t cli_realpath(const char *file_name, char **real_filename);
+
+#ifndef FREE
+#define FREE(VAR)      \
+    if (NULL != VAR) { \
+        free(VAR);     \
+        VAR = NULL;    \
+    }
+#endif /* FREE */
+
 #endif
